{*****************************************************************************
 *                                                                           *
 * Component: TDirectoryEdit                                                 *
 * Author: Brad Prendergast (bradp@bpsoftware.com) http://www.bpsoftware.com *
 * Date: September 14, 2000                                                  *
 * Version: 1.0                                                              *
 * Description: TEdit descendant that allows the selection of a directory    *
 *              to populate in the field.                                    *
 *                                                                           *
 * New:                                                                      *
 *  - Property SetHint: Boolean - Used to determine if hint is to be set to  *
 *                      selected directory                                   *
 *  - Event OnButtonClick: Event to occur if button is clicked - If set will *
 *                         override the directory selection call             *
 *                                                                           *
 *****************************************************************************}

unit DirectoryEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FileCtrl;

type
  TDirectoryEdit = class(TEdit)
  private
    { Private declarations }
    fButton: TButton;
    fSetHint: Boolean;
    fOnButtonClick: TNotifyEvent;
    procedure ChildButtonClick(Sender: TObject);
    procedure SetEditRect;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  protected
    { Protected declarations }
    function GetSetHint: Boolean;
    procedure SetSetHint(Value: Boolean);
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property OnButtonClick: TNotifyEvent read FOnButtonClick write FOnButtonClick;
    property SetHint: Boolean read GetSetHint write SetSetHint;
  end;

procedure Register;

implementation

constructor TDirectoryEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csCaptureMouse];
  AutoSize := False;
  fButton := TButton.Create(Self);
  fButton.Width  := 17;
  fButton.Height := 17;
  fButton.Visible := True;
  fButton.Parent := Self;
  fButton.SetBounds(0, 0, fButton.Width, fButton.Height);
  fButton.Caption := '...';
  FButton.OnClick := ChildButtonClick;
  Height := 21;
end;

destructor TDirectoryEdit.Destroy;
begin
  fButton := nil;
  inherited Destroy;
end;

procedure TDirectoryEdit.WMSize(var Message: TWMSize);
begin
  inherited;
  If FButton <> nil then
    begin
      if NewStyleControls and Ctl3D then
        FButton.SetBounds(Width - FButton.Width - 5, 0, FButton.Width, Height - 5)
      else
        FButton.SetBounds (Width - FButton.Width, 1, FButton.Width, Height - 3);
      SetEditRect;
    end;
end;

procedure TDirectoryEdit.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  SetEditRect;
end;

procedure TDirectoryEdit.CMTextChanged(var Message: TMessage);
begin
  inherited;
  If SetHint Then
    Hint := Text;
end;

procedure TDirectoryEdit.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  fButton.Enabled := Enabled;
end;

procedure TDirectoryEdit.ChildButtonClick(Sender: TObject);
var
  options : TSelectDirOpts;
  dir : string;
begin
  if Assigned(FOnButtonClick) then
    FOnButtonClick(Self)
  else
  Begin
    options := [sdAllowCreate, sdPerformCreate, sdPrompt];
    If SelectDirectory(dir, options,0) Then
      Text := Dir;
  End;
end;

procedure TDirectoryEdit.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end;

procedure TDirectoryEdit.SetEditRect;
var
  Loc: TRect;
begin
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
  Loc.Bottom := ClientHeight + 1;
  Loc.Right := ClientWidth - FButton.Width - 2;
  Loc.Top := 0;
  Loc.Left := 0;
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
end;

procedure TDirectoryEdit.SetSetHint(Value: Boolean);
begin
  fSetHint := Value;
end;

function TDirectoryEdit.GetSetHint: Boolean;
begin
  Result := fSetHint;
end;

procedure TDirectoryEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure Register;
begin
  RegisterComponents('NewComponents', [TDirectoryEdit]);
end;

end.
